#define I2C_BMP085 0x77

#include "Resources.h"

unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;

int _AC1, _AC2, _AC3, _B1, _B2, _MB, _MC, _MD;
unsigned int _AC4, _AC5, _AC6;
short _oss=0;
long _UT, _UP, _x1, _x2, _x3, _b3, _b5, _b6, _p, T, P;
unsigned long _b4, _b7;
float MMR, TT;

void DrawScr() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
}

void ReadCalibrationData() {
  short adress_index = 0;
  int result=0;
  char parameter[2]="";
  char date_[2]="";
  char temp[7];
  char temp_m[5];
  char temp_l[5];
  for (adress_index=0; adress_index<22; adress_index +=2){
    parameter[0]=0;
    parameter[1]=0;
    date_[0] = 0xAA + adress_index;
    I2C_Start();
    I2C_Write(I2C_BMP085,date_,1,END_MODE_RESTART);
    I2C_Read(I2C_BMP085,parameter,2,END_MODE_STOP);
    result = parameter[0]<<8 | parameter[1];
    switch (adress_index){
      case 0:  _AC1 = (int)result;
      case 2:  _AC2 = (int)result;
      case 4:  _AC3 = (int)result;
      case 6:  _AC4 = (unsigned int)result;
      case 8:  _AC5 = (unsigned int)result;
      case 10: _AC6 = (unsigned int)result;
      case 12: _B1  = (int)result;
      case 14: _B2  = (int)result;
      case 16: _MB  = (int)result;
      case 18: _MC  = (int)result;
      case 20: _MD  = (int)result;
      }
    }
  }

long ReadTemperature() {
  long result=0;
  char parameter[3]="";
  char date_[2]={0xF4,0x2E};
  I2C_Start();
  I2C_Write(0x77,date_,2,END_MODE_STOP);
  Delay_ms(5);
  date_[0]=0xF6;
  I2C_Start();
  I2C_Write(0x77,date_,1,END_MODE_RESTART);
  I2C_Read(0xF7,parameter,2,END_MODE_STOP);
  result = parameter[0]<<8 | parameter[1];
  return result;
}

int CalTemperature(long tval)
{
int result;
_x1=((_UT-_AC6)*_AC5) >> 15;
_x2=((long)_MC<<11)/(_x1+_MD);
_b5=_x1+_x2;
result=((_b5+8) >> 4);
return result;
}


long ReadPresure() {
  long result=0;
  char parameter[3]="";
  char date_[2] = {0xF4,0x34};
  switch (_oss){
  case 0: date_[2] = 0x34;
  case 1: date_[2] = 0x74;
  case 2: date_[2] = 0xB4;
  case 3: date_[2] = 0xF4;
  }
  I2C_Start();
  I2C_Write(0x77,date_,2,END_MODE_STOP);
  switch (_oss){
  case 0: Delay_ms(5);
  case 1: Delay_ms(8);
  case 2: Delay_ms(14);
  case 3: Delay_ms(26);
  }
  date_[0]=0xF6;
  I2C_Start();
  I2C_Write(0x77,date_,1,END_MODE_RESTART);
  I2C_Read(0xF7,parameter,3,END_MODE_STOP);
  result = parameter[0]<<8 | parameter[1];
  result <<= _oss;
  result |= parameter[2];
  return result;
}

long CalPresure(long pval)
{
_b6 = _b5-4000;

_x1 = (_b6*_b6)>>12;
_x1 *= _b2;
_x1 >>=11;

_x2 = _AC2*_b6;
_x2 >>= 11;

_x3 = _x1+_x2;

_b3 = (((((long)_AC1)*4+_x3)<<_oss)+2)>>2;

_x1 = (_AC3*_b6)>>13;

_x2 = (_b1*((_b6*_b6)>>12))>>16;

_x3 = ((_x1+_x2)+2)>>2;

_b4 = (_AC4*(unsigned long)(_x3+32768))>>15;

_b7 = ((unsigned long)(pval-_b3)*(50000>>_oss));

_p  = _b7<0x80000000 ? (_b7<<1)/_b4 : (_b7/_b4)<<1;

_x1 = _p>>8;
_x1 *= _x1;

_x1 = (_x1*3038)>>16;

_x2 = (_p*-7357)>>16;

_p += (_x1+_x2+3791)>>4;

return _p;
}

void main(){
  char temp1[10];
  I2C1_Init();
  I2C_Set_Active(&I2C1_Start, &I2C1_Read, &I2C1_Write);
  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);
  DrawScr();
  do{
    TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
    TFT_Set_Pen(CL_WHITE,1);
    TFT_Rectangle(10,10,120,90);
    ReadCalibrationData();
    _UT=ReadTemperature();
    T = CalTemperature(_UT);
    TFT_Write_Text("T: ", 10, 10);
    TT = (float)T/10.0;
    FloatToStr(TT, temp1);
    TFT_Write_Text(temp1, 50, 10);
    _UP=ReadPresure();
    P = CalPresure(_UP);
    LongToStr(P, temp1);
    TFT_Write_Text("P: ", 10, 30);
    TFT_Write_Text(temp1, 50, 30);
    MMR = (float)P*7.5006375541921/1000;
    TFT_Write_Text("MMR: ", 10, 50);
    FloatToStr(MMR, temp1);
    TFT_Write_Text(temp1, 50, 50);
    TFT_Write_Text("OSS: ", 10, 70);
    ShortToStr(_oss, temp1);
    TFT_Write_Text(temp1, 50, 70);
    delay_ms(1000);
  }while (1);
}